module Shift.Processing where

import Control.Lens ((<>~), (^.))
import Data.Default (def)
import Data.Git (Commit, Ref, commitMessage)
import Data.String.Conversions (cs)
import Data.Text (Text)
import Data.Void (Void)
import qualified Shift.Parsers as P
import Shift.Types
  ( ChangeReport,
    CommitType
      ( CTChore,
        CTDocs,
        CTFeature,
        CTFix,
        CTRefactor,
        CTStyle,
        CTTest
      ),
    ParsedCommit (PCConventional, PCMerge, PCMisc),
    ParsedGroup,
    ccType,
    crChores,
    crDocs,
    crFeatures,
    crFixes,
    crMerges,
    crMisc,
    crRefactors,
    crStyles,
    crTests,
  )
import Text.Megaparsec (ParseError, ParseErrorBundle, runParser)

parseCommit :: (Ref hash, Commit hash) -> Either (ParseErrorBundle Text Void) (ParsedGroup hash)
parseCommit (r, c) = do
  pc <- runParser P.commit "git" . cs . commitMessage $ c

  pure (r, c, pc)

generateReport :: [ParsedGroup hash] -> ChangeReport hash
generateReport = foldl inner def
  where
    inner acc (r, c, x) = case x of
      PCMisc miscCommit -> crMisc <>~ [(r, c, miscCommit)] $ acc
      PCMerge mergeCommit -> crMerges <>~ [(r, c, mergeCommit)] $ acc
      PCConventional commit ->
        ( case commit ^. ccType of
            CTFeature -> crFeatures
            CTFix -> crFixes
            CTDocs -> crDocs
            CTStyle -> crStyles
            CTRefactor -> crRefactors
            CTChore -> crChores
            CTTest -> crTests
        )
          <>~ [(r, c, commit)]
          $ acc
